/*****************************************************************************
 * Copyright (C) 2004
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace iOpener
{
	/// <summary>
	/// This class is a form for collecting an iTMS username and password.
	/// </summary>
	public class CredentialsDialog : System.Windows.Forms.Form
	{
		private	Label		label1;
		private Label		label2;
		private Label		label3;
		private TextBox		usernameTextBox;
		private TextBox		passwordTextBox;
		private Button		cancelButton;
		private Button		okButton;
		private GroupBox	groupBox1;
		private RadioButton	radioApple;
		private RadioButton	radioAOL;
		private Container	components = null;

		/// <summary>
		/// Constructor method.
		/// </summary>
		public CredentialsDialog()
		{
			InitializeComponent();
		}

		/// <summary>
		/// This method is the public accessor for the username and password vars
		/// </summary>
		/// <returns></returns>
		public DictionaryEntry GetCredentials()
		{
			return new DictionaryEntry(this.usernameTextBox.Text, this.passwordTextBox.Text);
		}

		/// <summary>
		/// This method is the public accessor for the type of login the user has (Apple or AOL currently)
		/// </summary>
		/// <returns></returns>
		public int GetLoginType()
		{
			return radioApple.Checked ? 0 : 1;
		}

		/// <summary>
		/// This method cleans up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(CredentialsDialog));
			this.label1 = new System.Windows.Forms.Label();
			this.usernameTextBox = new System.Windows.Forms.TextBox();
			this.passwordTextBox = new System.Windows.Forms.TextBox();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.cancelButton = new System.Windows.Forms.Button();
			this.okButton = new System.Windows.Forms.Button();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.radioApple = new System.Windows.Forms.RadioButton();
			this.radioAOL = new System.Windows.Forms.RadioButton();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(312, 64);
			this.label1.TabIndex = 0;
			this.label1.Text = "iOpener needs your iTunes Music Store credentials in order to retrieve the key to" +
				" iOpen your tracks.  This information will not be stored anywhere on your comput" +
				"er and will be sent only to the iTunes music store to retrieve your key.";
			// 
			// usernameTextBox
			// 
			this.usernameTextBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.usernameTextBox.Location = new System.Drawing.Point(88, 80);
			this.usernameTextBox.Name = "usernameTextBox";
			this.usernameTextBox.Size = new System.Drawing.Size(216, 20);
			this.usernameTextBox.TabIndex = 1;
			this.usernameTextBox.Text = "";
			// 
			// passwordTextBox
			// 
			this.passwordTextBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.passwordTextBox.Location = new System.Drawing.Point(88, 112);
			this.passwordTextBox.Name = "passwordTextBox";
			this.passwordTextBox.PasswordChar = '*';
			this.passwordTextBox.Size = new System.Drawing.Size(216, 20);
			this.passwordTextBox.TabIndex = 2;
			this.passwordTextBox.Text = "";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(8, 80);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(72, 23);
			this.label2.TabIndex = 3;
			this.label2.Text = "Username: ";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(8, 112);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(72, 23);
			this.label3.TabIndex = 4;
			this.label3.Text = "Password: ";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// cancelButton
			// 
			this.cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.cancelButton.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.cancelButton.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.cancelButton.Location = new System.Drawing.Point(167, 192);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.TabIndex = 8;
			this.cancelButton.Text = "&Cancel";
			// 
			// okButton
			// 
			this.okButton.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.okButton.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.okButton.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.okButton.Location = new System.Drawing.Point(79, 192);
			this.okButton.Name = "okButton";
			this.okButton.TabIndex = 7;
			this.okButton.Text = "&OK";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.radioAOL);
			this.groupBox1.Controls.Add(this.radioApple);
			this.groupBox1.Location = new System.Drawing.Point(24, 144);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(280, 40);
			this.groupBox1.TabIndex = 9;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Login Type";
			// 
			// radioApple
			// 
			this.radioApple.Checked = true;
			this.radioApple.Location = new System.Drawing.Point(72, 13);
			this.radioApple.Name = "radioApple";
			this.radioApple.Size = new System.Drawing.Size(56, 24);
			this.radioApple.TabIndex = 0;
			this.radioApple.TabStop = true;
			this.radioApple.Text = "Apple";
			// 
			// radioAOL
			// 
			this.radioAOL.Location = new System.Drawing.Point(160, 13);
			this.radioAOL.Name = "radioAOL";
			this.radioAOL.Size = new System.Drawing.Size(48, 24);
			this.radioAOL.TabIndex = 1;
			this.radioAOL.Text = "AOL";
			// 
			// CredentialsDialog
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(320, 230);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.cancelButton);
			this.Controls.Add(this.okButton);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.passwordTextBox);
			this.Controls.Add(this.usernameTextBox);
			this.Controls.Add(this.label1);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "CredentialsDialog";
			this.Text = "Please enter your iTMS credentials";
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion


	}
}
