/*****************************************************************************
 * FairStore.cs: FairStore 0.2
 *****************************************************************************
 * Copyright (C) 2004 Jon Lech Johansen <jon-vl@nanocrew.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

using System;
using System.IO;
using System.Net;
using System.Xml;
using System.Web;
using System.Text;
using System.Security.Cryptography;
using ICSharpCode.SharpZipLib.GZip;
using System.Collections.Specialized;

public class FairStore
{
    private string strToken;
    private string strDsid;
    private string strGUID;
    private NameValueCollection urlBag;

    public string Dsid
    {
        get
        {
            return( strDsid );
        }
    }

    public string GUID
    {
        get
        {
            return( strGUID );
        }

        set
        {
            strGUID = value;
        }
    }

    public static byte [] HexStringToBytes( string strHex )
    {
        byte [] bs = new byte[strHex.Length / 2];

        for( int i = 0; i < bs.Length; i++ )
        {
            bs[ i ] = Convert.ToByte( strHex.Substring( i * 2, 2 ), 16 );
        }

        return bs;
    }

    private string GetValueForKey( XmlTextReader xtr, string strKey )
    {
        if( String.Compare( xtr.Value, strKey, true ) == 0 )
        {
            int i = 0;

            while( xtr.Read() )
            {
                if( xtr.NodeType == XmlNodeType.Text )
                    return xtr.Value;
                if( i++ > 2 )
                    break;
            }
        }

        return null;
    }

    private NameValueCollection GetKeyValueList( XmlTextReader xtr,
                                                 string strName )
    {
        if( String.Compare( xtr.Value, strName, true ) == 0 )
        {
            bool bDict = false;
            NameValueCollection nvc = new NameValueCollection();

            while( xtr.Read() )
            {
                if( String.Compare( xtr.LocalName, "dict", true ) == 0 )
                {
                    if( bDict )
                        break;
                    bDict = !bDict;
                }

                if( xtr.NodeType == XmlNodeType.Text )
                {
                    int i = 0;
                    string strKey = xtr.Value;

                    while( xtr.Read() )
                    {
                        if( xtr.NodeType == XmlNodeType.Text )
                        {
                            nvc.Set( strKey, xtr.Value );
                            break;
                        }

                        if( i++ > 2 )
                            break;
                    }
                }
            }

            return nvc;
        }

        return null;
    }

    private string ComputeValidator( string strURL, string strUserAgent )
    {
        string strRandom = String.Format( "{0:X8}", new Random().Next() );
        byte [] abRandom = Encoding.ASCII.GetBytes( strRandom );

        byte [] abStatic =
            Convert.FromBase64String( "ROkjAaKid4EUF5kGtTNn3Q==" );

        int p = 0;
        for( int i = 0; i < 3 && p != -1; i++ )
            p = strURL.IndexOf( '/', p + 1 );

        byte [] abURL = Encoding.ASCII.GetBytes( strURL.Substring( p ) );
        byte [] abUA = Encoding.ASCII.GetBytes( strUserAgent );

        MD5CryptoServiceProvider MD5 = new MD5CryptoServiceProvider();
        MD5.TransformBlock( abURL, 0, abURL.Length, abURL, 0 );
        MD5.TransformBlock( abUA, 0, abUA.Length, abUA, 0 );
        MD5.TransformBlock( abStatic, 0, abStatic.Length, abStatic, 0 );
        MD5.TransformFinalBlock( abRandom, 0, abRandom.Length );

        return String.Format( "{0}-{1}", strRandom,
            BitConverter.ToString( MD5.Hash ).Replace( "-", "" ) );
    }

    private string Request( string strURL, string strParams )
    {
        HttpWebRequest req =
            (HttpWebRequest)WebRequest.Create( strURL + strParams );
        req.KeepAlive = false;

        req.Headers.Add( "Accept-Language: en-us, en;q=0.50" );

        DateTime now = DateTime.Now;
        TimeSpan ts = TimeZone.CurrentTimeZone.GetUtcOffset( now );
        String strTz = String.Format( "X-Apple-Tz: {0}", ts.Hours * 3600 );
        req.Headers.Add( strTz );

        if( strToken != null )
            req.Headers.Add( "X-Token: " + strToken );

        req.UserAgent = "iTunes/4.6 (Macintosh; U; PPC Mac OS X 10.3)";

        if( strDsid != null )
            req.Headers.Add( "X-Dsid: " + strDsid );

        req.Headers.Add( "X-Apple-Validation: " +
                         ComputeValidator( strURL, req.UserAgent ) );

        req.Headers.Add( "Accept-Encoding: gzip, x-aes-cbc" );

        HttpWebResponse res = (HttpWebResponse)req.GetResponse();

        byte [] content = new byte [ res.ContentLength ];
        res.GetResponseStream().Read( content, 0, content.Length );

        if( res.ContentEncoding.IndexOf( "x-aes-cbc" ) != -1 )
        {
            Rijndael alg;

            alg = Rijndael.Create();
            alg.Mode = CipherMode.CBC;
            alg.Padding = PaddingMode.None;

            MemoryStream cms = new MemoryStream();

            byte [] Key =
                Convert.FromBase64String( "mNHiLKoNir1l0UOtJ1pe5w==" );

            string strProKey = res.Headers[ "x-apple-protocol-key" ];
            if( Convert.ToUInt32( strProKey ) != 3 )
                throw new Exception( "Dec failed (ProKey != 3)" );

            string strIV = res.Headers[ "x-apple-crypto-iv" ];
            byte [] IV = HexStringToBytes( strIV );

            ICryptoTransform ct = alg.CreateDecryptor( Key, IV );
            CryptoStream cs =
                new CryptoStream( cms, ct, CryptoStreamMode.Write );
            cs.Write( content, 0, (content.Length / 16) * 16 );
            cs.Close();

            content = cms.ToArray();
        }

        if( res.ContentEncoding.IndexOf( "gzip" ) != -1 )
        {
            MemoryStream gms = new MemoryStream( content, false );
            GZipInputStream gzi = new GZipInputStream( gms );
            StreamReader sr = new StreamReader( gzi );
            content = Encoding.UTF8.GetBytes( sr.ReadToEnd() );
        }

        res.Close();

        return Encoding.UTF8.GetString( content );
    }

    public void Login( string strUserID, string strPassword,
                       int AccountKind )
    {
        string rs;
        StringReader sr;
        XmlTextReader xtr;

        if( strGUID == null )
        {
            throw new Exception( "Login error (GUID is null)" );
        }

        Request( "http://phobos.apple.com/storeBag.xml.gz", null );
        rs = Request( "http://phobos.apple.com/secureBag.xml.gz", null );

        sr = new StringReader( rs );
        xtr = new XmlTextReader( sr );
        while( xtr.Read() )
        {
            if( xtr.NodeType == XmlNodeType.Text )
            {
                if( urlBag == null )
                    urlBag = GetKeyValueList( xtr, "urlBag" );
            }
        }
        xtr.Close();

        if( urlBag.Get( "authenticateAccount" ) == null )
            throw new Exception( "authenticateAccount not in urlBag" );

        string strParams = String.Format( "?appleId={0}&password={1}" +
            "&accountKind={2}&attempt=1&guid={3}",
            HttpUtility.UrlEncode( strUserID ),
            HttpUtility.UrlEncode( strPassword ),
            AccountKind, HttpUtility.UrlEncode( strGUID ) );

        rs = Request( urlBag.Get( "authenticateAccount" ), strParams );

        string strDocType = null;
        string strCusMsg = null;
        sr = new StringReader( rs );
        xtr = new XmlTextReader( sr );
        while( xtr.Read() )
        {
            if( xtr.NodeType == XmlNodeType.Text )
            {
                if( strDocType == null )
                    strDocType = GetValueForKey( xtr, "jingleDocType" );
                if( strCusMsg == null )
                    strCusMsg = GetValueForKey( xtr, "customerMessage" );
                if( strToken == null )
                    strToken = GetValueForKey( xtr, "passwordToken" );
                if( strDsid == null )
                    strDsid = GetValueForKey( xtr, "dsPersonId" );
            }
        }
        xtr.Close();

        if( String.Compare( strDocType,
                            "authenticationSuccess", true ) != 0 )
        {
            throw new Exception( String.Format( "Login failed ({0})",
                                                strCusMsg ) );
        }

        if( strToken == null || strDsid == null )
            throw new Exception( "Login error (Token/Dsid is null)" );
    }

    public NameValueCollection AuthorizeMachine()
    {
        NameValueCollection nvc = null;

        if( urlBag.Get( "authorizeMachine" ) == null )
            throw new Exception( "authorizeMachine not in urlBag" );

        string strParams = String.Format( "?guid={0}",
            HttpUtility.UrlEncode( strGUID ) );

        string rs = Request( urlBag.Get( "authorizeMachine" ), strParams );

        string strDocType = null;
        string strCusMsg = null;
        StringReader sr = new StringReader( rs );
        XmlTextReader xtr = new XmlTextReader( sr );
        while( xtr.Read() )
        {
            if( xtr.NodeType == XmlNodeType.Text )
            {
                if( strDocType == null )
                    strDocType = GetValueForKey( xtr, "jingleDocType" );
                if( strCusMsg == null )
                    strCusMsg = GetValueForKey( xtr, "customerMessage" );
                if( nvc == null )
                    nvc = GetKeyValueList( xtr, "encryptionKeys" );
            }
        }
        xtr.Close();

        if( String.Compare( strDocType, "machineAuthorizationInfoSuccess",
                            true ) != 0 )
        {
            throw new Exception( String.Format(
                "AuthorizeMachine failed ({0})", strCusMsg ) );
        }

        return nvc;
    }

    public void DeauthorizeMachine()
    {
        if( urlBag.Get( "deauthorizeMachine" ) == null )
            throw new Exception( "deauthorizeMachine not in urlBag" );

        string strParams = String.Format( "?guid={0}",
            HttpUtility.UrlEncode( strGUID ) );

        string rs = Request( urlBag.Get( "deauthorizeMachine" ), strParams );

        string strDocType = null;
        string strCusMsg = null;
        StringReader sr = new StringReader( rs );
        XmlTextReader xtr = new XmlTextReader( sr );
        while( xtr.Read() )
        {
            if( xtr.NodeType == XmlNodeType.Text )
            {
                if( strDocType == null )
                    strDocType = GetValueForKey( xtr, "jingleDocType" );
                if( strCusMsg == null )
                    strCusMsg = GetValueForKey( xtr, "customerMessage" );
            }
        }
        xtr.Close();

        if( String.Compare( strDocType, "success", true ) != 0 )
        {
            throw new Exception( String.Format(
                "DeauthorizeMachine failed ({0})", strCusMsg ) );
        }
    }
}
