/*****************************************************************************
 * Copyright (C) 2004
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;
using iTunesLib;

/// TODO: deal with form resizing

namespace iOpener
{
	/// <summary>
	/// This class is the main form for iOpener
	/// </summary>
	public class iOpenerForm : System.Windows.Forms.Form
	{
		private iOpenerApplicationContext		applicationContext;					// the app context controller class instance
		private bool							lastLogMessageTransient = false;	// for tracking whether we need to keep the last logged message
		private Button							iOpenButton;
		private ToolTip							iOpenToolTip;
		private Label							helpLabel;
		private ListBox							statusListBox;
		private Button							prefsButton;
		private IContainer						components;

		/// <summary>
		/// This is the constructor method which inits all components and sets the app context
		/// for future reference.
		/// </summary>
		/// <param name="context"></param>
		public iOpenerForm(iOpenerApplicationContext context)
		{
			InitializeComponent();
			this.applicationContext = context;
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(iOpenerForm));
			this.statusListBox = new System.Windows.Forms.ListBox();
			this.iOpenButton = new System.Windows.Forms.Button();
			this.iOpenToolTip = new System.Windows.Forms.ToolTip(this.components);
			this.prefsButton = new System.Windows.Forms.Button();
			this.helpLabel = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// statusListBox
			// 
			this.statusListBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.statusListBox.Location = new System.Drawing.Point(9, 8);
			this.statusListBox.Name = "statusListBox";
			this.statusListBox.Size = new System.Drawing.Size(319, 210);
			this.statusListBox.TabIndex = 1;
			this.iOpenToolTip.SetToolTip(this.statusListBox, "Information on the current iOpen operation");
			// 
			// iOpenButton
			// 
			this.iOpenButton.BackColor = System.Drawing.Color.Transparent;
			this.iOpenButton.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.iOpenButton.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.iOpenButton.ForeColor = System.Drawing.SystemColors.ControlText;
			this.iOpenButton.Location = new System.Drawing.Point(76, 232);
			this.iOpenButton.Name = "iOpenButton";
			this.iOpenButton.TabIndex = 3;
			this.iOpenButton.Text = "iOpen!";
			this.iOpenToolTip.SetToolTip(this.iOpenButton, "Decrypt my entire iTunes library");
			this.iOpenButton.Click += new System.EventHandler(this.iOpenButton_Click);
			// 
			// prefsButton
			// 
			this.prefsButton.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.prefsButton.Font = new System.Drawing.Font("Tahoma", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.prefsButton.Location = new System.Drawing.Point(188, 232);
			this.prefsButton.Name = "prefsButton";
			this.prefsButton.TabIndex = 5;
			this.prefsButton.Text = "&Prefs";
			this.iOpenToolTip.SetToolTip(this.prefsButton, "Set my preferences");
			this.prefsButton.Click += new System.EventHandler(this.prefsButton_Click);
			// 
			// helpLabel
			// 
			this.helpLabel.ForeColor = System.Drawing.SystemColors.ControlDarkDark;
			this.helpLabel.Location = new System.Drawing.Point(33, 264);
			this.helpLabel.Name = "helpLabel";
			this.helpLabel.Size = new System.Drawing.Size(272, 32);
			this.helpLabel.TabIndex = 4;
			this.helpLabel.Text = "Pressing the iOpen button will iOpen (decrypt) your entire iTunes library.";
			this.helpLabel.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// iOpenerForm
			// 
			this.AcceptButton = this.iOpenButton;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(338, 304);
			this.Controls.Add(this.prefsButton);
			this.Controls.Add(this.helpLabel);
			this.Controls.Add(this.iOpenButton);
			this.Controls.Add(this.statusListBox);
			this.ForeColor = System.Drawing.SystemColors.WindowText;
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.Name = "iOpenerForm";
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
			this.Text = "iOpener";
			this.SizeChanged += new System.EventHandler(this.iOpenerForm_SizeChanged);
			this.ResumeLayout(false);

		}
		#endregion
		/// <summary>
		/// This method is passed as a delegate to handle logging for the iOpener class
		/// </summary>
		/// <param name="message"></param>
		public void Logger(string message, bool transient) 
		{
			if (lastLogMessageTransient && statusListBox.Items.Count > 0)
			{
				statusListBox.Items[0] = message;
			} 
			else 
			{
				statusListBox.Items.Insert(0, message);
			}
			lastLogMessageTransient = transient;
			statusListBox.Refresh();
		}

		/// <summary>
		/// This method allows other classes to log things to this form if it's open.
		/// </summary>
		/// <returns></returns>
		public iOpener.LogHandler GetLogHandler() 
		{
			return new iOpener.LogHandler(Logger);
		}

		/// <summary>
		/// This method cleans up subcomponents prior to being disposed of.
		/// </summary>
		/// <param name="disposing"></param>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// Because we have a notify icon, we simply close this form on minimize.  This keeps
		/// it out of the taskbar.  It will get recreated via the notify icon's menu if needed
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void iOpenerForm_SizeChanged(object sender, System.EventArgs e)
		{
			if (this.WindowState == FormWindowState.Minimized) 
			{
				this.Close();
			}
		}

		/// <summary>
		/// This method uses an instance of the iOpener class to decrypt the entire
		/// iTunes library.  It launches the DecryptLibrary fxn in another thread
		/// so the UI remains responsive.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void iOpenButton_Click(object sender, System.EventArgs e)
		{
			applicationContext.InitializeItunesApp();
			statusListBox.Items.Clear();
			iOpener io = new iOpener((iTunesApp)applicationContext.GetItunesApp(), applicationContext);
			iOpener.LogHandler lh = new iOpener.LogHandler(Logger);
			try 
			{
				DecryptLibraryParameters p = new DecryptLibraryParameters(lh, applicationContext.ShouldCopyFiles());
				ThreadPool.QueueUserWorkItem(new WaitCallback(io.DecryptLibrary), p);
			}
			catch (KeyRetrievalException kr) 
			{
				MessageBox.Show(this, "It appears you have authorized too many computers.  Please go to http://www.apple.com/support/itunes/authorization.html to try deauthorizing one of your computers.");
			}
		}

		/// <summary>
		/// This method is the event handler for clicking the prefs button
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void prefsButton_Click(object sender, System.EventArgs e)
		{
			applicationContext.ShowPrefs();
		}

	}
}
